/*
 * AMD GCN ISA Assembler
 *
 * GCN ISA instruction structures
 *
 * This software is Copyright 2013, Daniel Bali <balijanosdaniel at gmail.com>,
 * and it is hereby released to the general public under the following terms:
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted.
 */

#include "isa_instr.h"

const isa_instr isa_instr_list[] =
{
	// SOP2 instructions
	{"S_ADD_U32",					0x80000000, SOP2, NONE, 0},
	{"S_SUB_U32",					0x80800000, SOP2, NONE, 0},
	{"S_ADD_I32",					0x81000000, SOP2, NONE, 0},
	{"S_SUB_I32",					0x81800000, SOP2, NONE, 0},
	{"S_ADDC_U32",					0x82000000, SOP2, NONE, 0},
	{"S_SUBB_U32",					0x82800000, SOP2, NONE, 0},
	{"S_MIN_I32",					0x83000000, SOP2, NONE, 0},
	{"S_MIN_U32",					0x83800000, SOP2, NONE, 0},
	{"S_MAX_I32",					0x84000000, SOP2, NONE, 0},
	{"S_MAX_U32",					0x84800000, SOP2, NONE, 0},
	{"S_CSELECT_B32",				0x85000000, SOP2, NONE, 0},
	{"S_CSELECT_B64",				0x85800000, SOP2, NONE, 0},
	{"S_AND_B32",					0x87000000, SOP2, NONE, 0},
	{"S_AND_B64",					0x87800000, SOP2, NONE, 0},
	{"S_OR_B32",					0x88000000, SOP2, NONE, 0},
	{"S_OR_B64",					0x88800000, SOP2, NONE, 0},
	{"S_XOR_B32",					0x89000000, SOP2, NONE, 0},
	{"S_XOR_B64",					0x89800000, SOP2, NONE, 0},
	{"S_ANDN2_B32",					0x8A000000, SOP2, NONE, 0},
	{"S_ANDN2_B64",					0x8A800000, SOP2, NONE, 0},
	{"S_ORN2_B32",					0x8B000000, SOP2, NONE, 0},
	{"S_ORN2_B64",					0x8B800000, SOP2, NONE, 0},
	{"S_NAND_B32",					0x8C000000, SOP2, NONE, 0},
	{"S_NAND_B64",					0x8C800000, SOP2, NONE, 0},
	{"S_NOR_B32",					0x8D000000, SOP2, NONE, 0},
	{"S_NOR_B64",					0x8D800000, SOP2, NONE, 0},
	{"S_XNOR_B32",					0x8E000000, SOP2, NONE, 0},
	{"S_XNOR_B64",					0x8E800000, SOP2, NONE, 0},
	{"S_LSHL_B32",					0x8F000000, SOP2, NONE, 0},
	{"S_LSHL_B64",					0x8F800000, SOP2, NONE, 0},
	{"S_LSHR_B32",					0x90000000, SOP2, NONE, 0},
	{"S_LSHR_B64",					0x90800000, SOP2, NONE, 0},
	{"S_ASHR_I32",					0x91000000, SOP2, NONE, 0},
	{"S_ASHR_I64",					0x91800000, SOP2, NONE, 0},
	{"S_BFM_B32",					0x92000000, SOP2, NONE, 0},
	{"S_BFM_B64",					0x92800000, SOP2, NONE, 0},
	{"S_MUL_I32",					0x93000000, SOP2, NONE, 0},
	{"S_BFE_U32",					0x93800000, SOP2, NONE, 0},
	{"S_BFE_I32",					0x94000000, SOP2, NONE, 0},
	{"S_BFE_U64",					0x94800000, SOP2, NONE, 0},
	{"S_BFE_I64",					0x95000000, SOP2, NONE, 0},
	{"S_CBRANCH_G_FORK",			0x95800000, SOP2, NONE, 0},
	{"S_ABSDIFF_I32",				0x96000000, SOP2, NONE, 0},

	// SOPK instructions
	{"S_MOVK_I32",					0xB0000000, SOPK, NONE, 0},
	{"S_CMOVK_I32",					0xB1000000, SOPK, NONE, 0},
	{"S_CMPK_EQ_I32",				0xB1800000, SOPK, NONE, 0},
	{"S_CMPK_LG_I32",				0xB2000000, SOPK, NONE, 0},
	{"S_CMPK_GT_I32",				0xB2800000, SOPK, NONE, 0},
	{"S_CMPK_GE_I32",				0xB3000000, SOPK, NONE, 0},
	{"S_CMPK_LT_I32",				0xB3800000, SOPK, NONE, 0},
	{"S_CMPK_LE_I32",				0xB4000000, SOPK, NONE, 0},
	{"S_CMPK_EQ_U32",				0xB4800000, SOPK, NONE, 0},
	{"S_CMPK_LG_U32",				0xB5000000, SOPK, NONE, 0},
	{"S_CMPK_GT_U32",				0xB5800000, SOPK, NONE, 0},
	{"S_CMPK_GE_U32",				0xB6000000, SOPK, NONE, 0},
	{"S_CMPK_LT_U32",				0xB6800000, SOPK, NONE, 0},
	{"S_CMPK_LE_U32",				0xB7000000, SOPK, NONE, 0},
	{"S_ADDK_I32",					0xB7800000, SOPK, NONE, 0},
	{"S_MULK_I32",					0xB8000000, SOPK, NONE, 0},
	{"S_CBRANCH_I_FORK",			0xB8800000, SOPK, NONE, 0},
	{"S_GETREG_B32",				0xB9000000, SOPK, NONE, 0},
	{"S_SETREG_B32",				0xB9800000, SOPK, NONE, 0},
	{"S_SETREG_IMM32_B32",			0xBA800000, SOPK, NONE, 0},

	// SOP1 instructions
	{"S_MOV_B32",					0xBE800300,	SOP1, NONE, 0},
	{"S_MOV_B64",					0xBE800400,	SOP1, NONE, 0},
	{"S_CMOV_B32",					0xBE800500,	SOP1, NONE, 0},
	{"S_CMOV_B64",					0xBE800600,	SOP1, NONE, 0},
	{"S_NOT_B32",					0xBE800700,	SOP1, NONE, 0},
	{"S_NOT_B64",					0xBE800800,	SOP1, NONE, 0},
	{"S_WQM_B32",					0xBE800900,	SOP1, NONE, 0},
	{"S_WQM_B64	",					0xBE800A00,	SOP1, NONE, 0},
	{"S_BREV_B32",					0xBE800B00,	SOP1, NONE, 0},
	{"S_BREV_B64",					0xBE800C00,	SOP1, NONE, 0},
	{"S_BCNT0_I32_B32",				0xBE800D00,	SOP1, NONE, 0},
	{"S_BCNT0_I32_B64",				0xBE800E00,	SOP1, NONE, 0},
	{"S_BCNT1_I32_B32",				0xBE800F00,	SOP1, NONE, 0},
	{"S_BCNT1_I32_B64",				0xBE801000,	SOP1, NONE, 0},
	{"S_FF0_I32_B32",				0xBE801100,	SOP1, NONE, 0},
	{"S_FF0_I32_B64",				0xBE801200,	SOP1, NONE, 0},
	{"S_FF1_I32_B32",				0xBE801300,	SOP1, NONE, 0},
	{"S_FF1_I32_B64",				0xBE801400,	SOP1, NONE, 0},
	{"S_FLBIT_I32_B32",				0xBE801500,	SOP1, NONE, 0},
	{"S_FLBIT_I32_B64",				0xBE801600,	SOP1, NONE, 0},
	{"S_FLBIT_I32",					0xBE801700,	SOP1, NONE, 0},
	{"S_FLBIT_I32_I64",				0xBE801800,	SOP1, NONE, 0},
	{"S_SEXT_I32_I8",				0xBE801900,	SOP1, NONE, 0},
	{"S_SEXT_I32_I16",				0xBE801A00,	SOP1, NONE, 0},
	{"S_BITSET0_B32",				0xBE801B00,	SOP1, NONE, 0},
	{"S_BITSET0_B64",				0xBE801C00,	SOP1, NONE, 0},
	{"S_BITSET1_B32",				0xBE801D00,	SOP1, NONE, 0},
	{"S_BITSET1_B64",				0xBE801E00,	SOP1, NONE, 0},
	{"S_GETPC_B64",					0xBE801F00,	SOP1, NONE, 0},
	{"S_SETPC_B64",					0xBE802000,	SOP1, NONE, 0},
	{"S_SWAPPC_B64",				0xBE802100,	SOP1, NONE, 0},
	{"S_RFE_B64",					0xBE802200,	SOP1, NONE, 0},
	{"S_AND_SAVEEXEC_B64",			0xBE802400,	SOP1, NONE, 0},
	{"S_OR_SAVEEXEC_B64",			0xBE802500,	SOP1, NONE, 0},
	{"S_XOR_SAVEEXEC_B64",			0xBE802600,	SOP1, NONE, 0},
	{"S_ANDN2_SAVEEXEC_B64",		0xBE802700,	SOP1, NONE, 0},
	{"S_ORN2_SAVEEXEC_B64",			0xBE802800,	SOP1, NONE, 0},
	{"S_NAND_SAVEEXEC_B64",			0xBE802900,	SOP1, NONE, 0},
	{"S_NOR_SAVEEXEC_B64",			0xBE802A00,	SOP1, NONE, 0},
	{"S_XNOR_SAVEEXEC_B64",			0xBE802B00,	SOP1, NONE, 0},
	{"S_QUADMASK_B32",				0xBE802C00,	SOP1, NONE, 0},
	{"S_QUADMASK_B64",				0xBE802D00,	SOP1, NONE, 0},
	{"S_MOVRELS_B32",				0xBE802E00,	SOP1, NONE, 0},
	{"S_MOVRELS_B64",				0xBE802F00,	SOP1, NONE, 0},
	{"S_MOVRELD_B32",				0xBE803000,	SOP1, NONE, 0},
	{"S_MOVRELD_B64",				0xBE803100,	SOP1, NONE, 0},
	{"S_CBRANCH_JOIN",				0xBE803200,	SOP1, NONE, 0},
	{"S_ABS_I32",					0xBE803400,	SOP1, NONE, 0},
	{"S_MOV_FED_B32",				0xBE803500,	SOP1, NONE, 0},

	// SOPC instructions
	{"S_CMP_EQ_I32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_LG_I32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_GT_I32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_GE_I32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_LT_I32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_LE_I32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_EQ_U32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_LG_U32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_GT_U32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_GE_U32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_LT_U32",				0xBF000000,	SOPC, NONE, 0},
	{"S_CMP_LE_U32",				0xBF000000,	SOPC, NONE, 0},
	{"S_BITCMP0_B32",				0xBF000000,	SOPC, NONE, 0},
	{"S_BITCMP1_B32",				0xBF000000,	SOPC, NONE, 0},
	{"S_BITCMP0_B64",				0xBF000000,	SOPC, NONE, 0},
	{"S_BITCMP1_B64",				0xBF000000,	SOPC, NONE, 0},
	{"S_SETVSKIP",					0xBF000000,	SOPC, NONE, 0},

	// SOPP instructions
	{"S_NOP",						0xBF800000, SOPP, NONE, 0},
	{"S_ENDPGM",					0xBF810000, SOPP, NONE, 0},
	{"S_BRANCH",					0xBF820000, SOPP, NONE, 0},
	{"S_CBRANCH_SCC0",				0xBF840000, SOPP, NONE, 0},
	{"S_CBRANCH_SCC1",				0xBF850000, SOPP, NONE, 0},
	{"S_CBRANCH_VCCZ",				0xBF860000, SOPP, NONE, 0},
	{"S_CBRANCH_VCCNZ",				0xBF870000, SOPP, NONE, 0},
	{"S_CBRANCH_EXECZ",				0xBF880000, SOPP, NONE, 0},
	{"S_CBRANCH_EXECNZ",			0xBF890000, SOPP, NONE, 0},
	{"S_BARRIER",					0xBF8A0000, SOPP, NONE, 0},
	{"S_WAITCNT",					0xBF8C0000, SOPP, NONE, 0},
	{"S_SETHALT",					0xBF8D0000, SOPP, NONE, 0},
	{"S_SLEEP",						0xBF8E0000, SOPP, NONE, 0},
	{"S_SETPRIO",					0xBF8F0000, SOPP, NONE, 0},
	{"S_SENDMSG",					0xBF900000, SOPP, NONE, 0},
	{"S_SENDMSGHALT",				0xBF910000, SOPP, NONE, 0},
	{"S_TRAP",						0xBF920000, SOPP, NONE, 0},
	{"S_ICACHE_INV",				0xBF930000, SOPP, NONE, 0},
	{"S_INCPERFLEVEL",				0xBF940000, SOPP, NONE, 0},
	{"S_DECPERFLEVEL",				0xBF950000, SOPP, NONE, 0},
	{"S_TTRACEDATA",				0xBF960000, SOPP, NONE, 0},

	// SMRD instructions
	{"S_LOAD_DWORD",				0xC0000000, SMRD, NONE, 0},
	{"S_LOAD_DWORDX2",				0xC0400000, SMRD, NONE, 0},
	{"S_LOAD_DWORDX4",				0xC0800000, SMRD, NONE, 0},
	{"S_LOAD_DWORDX8",				0xC0C00000, SMRD, NONE, 0},
	{"S_LOAD_DWORDX16",				0xC1000000, SMRD, NONE, 0},
	{"S_BUFFER_LOAD_DWORD",			0xC2000000, SMRD, NONE, 0},
	{"S_BUFFER_LOAD_DWORDX2",		0xC2400000, SMRD, NONE, 0},
	{"S_BUFFER_LOAD_DWORDX4",		0xC2800000, SMRD, NONE, 0},
	{"S_BUFFER_LOAD_DWORDX8",		0xC2C00000, SMRD, NONE, 0},
	{"S_BUFFER_LOAD_DWORDX16",		0xC3000000, SMRD, NONE, 0},
	{"S_MEMTIME",					0xC7800000, SMRD, NONE, 0},
	{"S_DCACHE_INV",				0xC7C00000, SMRD, NONE, 0},

	// VOP2 instructions
	{"V_CNDMASK_B32",				0x00000000, VOP2, VOP3a, 0xD2000000},
	{"V_READLANE_B32",				0x02000000, VOP2, VOP3a, 0xD2020000},
	{"V_WRITELANE_B32",				0x04000000, VOP2, VOP3a, 0xD2040000},
	{"V_ADD_F32",					0x06000000, VOP2, VOP3a, 0xD2060000},
	{"V_SUB_F32",					0x08000000, VOP2, VOP3a, 0xD2080000},
	{"V_SUBREV_F32",				0x0A000000, VOP2, VOP3a, 0xD20A0000},
	{"V_MAC_LEGACY_F32",			0x0C000000, VOP2, VOP3a, 0xD20C0000},
	{"V_MUL_LEGACY_F32",			0x0E000000, VOP2, VOP3a, 0xD20E0000},
	{"V_MUL_F32",					0x10000000, VOP2, VOP3a, 0xD2100000},
	{"V_MUL_I32_I24",				0x12000000, VOP2, VOP3a, 0xD2120000},
	{"V_MUL_HI_I32_I24",			0x14000000, VOP2, VOP3a, 0xD2140000},
	{"V_MUL_U32_U24",				0x16000000, VOP2, VOP3a, 0xD2160000},
	{"V_MUL_HI_U32_U24",			0x18000000, VOP2, VOP3a, 0xD2180000},
	{"V_MIN_LEGACY_F32",			0x1A000000, VOP2, VOP3a, 0xD21A0000},
	{"V_MAX_LEGACY_F32",			0x1C000000, VOP2, VOP3a, 0xD21C0000},
	{"V_MIN_F32",					0x1E000000, VOP2, VOP3a, 0xD21E0000},
	{"V_MAX_F32",					0x20000000, VOP2, VOP3a, 0xD2200000},
	{"V_MIN_I32",					0x22000000, VOP2, VOP3a, 0xD2220000},
	{"V_MAX_I32",					0x24000000, VOP2, VOP3a, 0xD2240000},
	{"V_MIN_U32",					0x26000000, VOP2, VOP3a, 0xD2260000},
	{"V_MAX_U32",					0x28000000, VOP2, VOP3a, 0xD2280000},
	{"V_LSHR_B32",					0x2A000000, VOP2, VOP3a, 0xD22A0000},
	{"V_LSHRREV_B32",				0x2C000000, VOP2, VOP3a, 0xD22C0000},
	{"V_ASHR_I32",					0x2E000000, VOP2, VOP3a, 0xD22E0000},
	{"V_ASHRREV_I32",				0x30000000, VOP2, VOP3a, 0xD2300000},
	{"V_LSHL_B32",					0x32000000, VOP2, VOP3a, 0xD2320000},
	{"V_LSHLREV_B32",				0x34000000, VOP2, VOP3a, 0xD2340000},
	{"V_AND_B32",					0x36000000, VOP2, VOP3a, 0xD2360000},
	{"V_OR_B32",					0x38000000, VOP2, VOP3a, 0xD2380000},
	{"V_XOR_B32",					0x3A000000, VOP2, VOP3a, 0xD23A0000},
	{"V_BFM_B32",					0x3C000000, VOP2, VOP3a, 0xD23C0000},
	{"V_MAC_F32",					0x3E000000, VOP2, VOP3a, 0xD23E0000},
	{"V_MADMK_F32",					0x40000000, VOP2, VOP3a, 0xD2400000},
	{"V_MADAK_F32",					0x42000000, VOP2, VOP3a, 0xD2420000},
	{"V_BCNT_U32_B32",				0x44000000, VOP2, VOP3a, 0xD2440000},
	{"V_MBCNT_LO_U32_B32",			0x46000000, VOP2, VOP3a, 0xD2460000},
	{"V_MBCNT_HI_U32_B32",			0x48000000, VOP2, VOP3a, 0xD2480000},
	{"V_ADD_I32",					0x4A000000, VOP2, VOP3b, 0xD24A0000},
	{"V_SUB_I32",					0x4C000000, VOP2, VOP3b, 0xD24C0000},
	{"V_SUBREV_I32",				0x4E000000, VOP2, VOP3b, 0xD24E0000},
	{"V_ADDC_U32",					0x50000000, VOP2, VOP3b, 0xD2500000},
	{"V_SUBB_U32",					0x52000000, VOP2, VOP3b, 0xD2520000},
	{"V_SUBBREV_U32",				0x54000000, VOP2, VOP3b, 0xD2540000},
	{"V_LDEXP_F32",					0x56000000, VOP2, VOP3a, 0xD2560000},
	{"V_CVT_PKACCUM_U8_F32",		0x58000000, VOP2, VOP3a, 0xD2580000},
	{"V_CVT_PKNORM_I16_F32",		0x5A000000, VOP2, VOP3a, 0xD25A0000},
	{"V_CVT_PKNORM_U16_F32",		0x5C000000, VOP2, VOP3a, 0xD25C0000},
	{"V_CVT_PKRTZ_F16_F32",			0x5E000000, VOP2, VOP3a, 0xD25E0000},
	{"V_CVT_PK_U16_U32",			0x60000000, VOP2, VOP3a, 0xD2600000},
	{"V_CVT_PK_I16_I32",			0x62000000, VOP2, VOP3a, 0xD2620000},

	// VOP1 instructions
	{"V_NOP",						0x7E000000,	VOP1, VOP3a, 0XD3000000},
	{"V_MOV_B32",					0x7E000200,	VOP1, VOP3a, 0XD3020000},
	{"V_READFIRSTLANE_B32",			0x7E000400,	VOP1, VOP3a, 0XD3040000},
	{"V_CVT_I32_F64",				0x7E000600,	VOP1, VOP3a, 0XD3060000},
	{"V_CVT_F64_I32",				0x7E000800,	VOP1, VOP3a, 0XD3080000},
	{"V_CVT_F32_I32",				0x7E000A00,	VOP1, VOP3a, 0XD30A0000},
	{"V_CVT_F32_U32",				0x7E000C00,	VOP1, VOP3a, 0XD30C0000},
	{"V_CVT_U32_F32",				0x7E000E00,	VOP1, VOP3a, 0XD30E0000},
	{"V_CVT_I32_F32",				0x7E001000,	VOP1, VOP3a, 0XD3100000},
	{"V_MOV_FED_B32",				0x7E001200,	VOP1, VOP3a, 0XD3120000},
	{"V_CVT_F16_F32",				0x7E001400,	VOP1, VOP3a, 0XD3140000},
	{"V_CVT_F32_F16",				0x7E001600,	VOP1, VOP3a, 0XD3160000},
	{"V_CVT_RPI_I32_F32",			0x7E001800,	VOP1, VOP3a, 0XD3180000},
	{"V_CVT_FLR_I32_F32",			0x7E001A00,	VOP1, VOP3a, 0XD31A0000},
	{"V_CVT_OFF_F32_I4",			0x7E001C00,	VOP1, VOP3a, 0XD31C0000},
	{"V_CVT_F32_F64",				0x7E001E00,	VOP1, VOP3a, 0XD31E0000},
	{"V_CVT_F64_F32",				0x7E002000,	VOP1, VOP3a, 0XD3200000},
	{"V_CVT_F32_UBYTE0",			0x7E002200,	VOP1, VOP3a, 0XD3220000},
	{"V_CVT_F32_UBYTE1",			0x7E002400,	VOP1, VOP3a, 0XD3240000},
	{"V_CVT_F32_UBYTE2",			0x7E002600,	VOP1, VOP3a, 0XD3260000},
	{"V_CVT_F32_UBYTE3",			0x7E002800,	VOP1, VOP3a, 0XD3280000},
	{"V_CVT_U32_F64",				0x7E002A00,	VOP1, VOP3a, 0XD32A0000},
	{"V_CVT_F64_U32",				0x7E002C00,	VOP1, VOP3a, 0XD32C0000},
	{"V_FRACT_F32",					0x7E004000,	VOP1, VOP3a, 0XD3400000},
	{"V_TRUNC_F32",					0x7E004200,	VOP1, VOP3a, 0XD3420000},
	{"V_CEIL_F32",					0x7E004400,	VOP1, VOP3a, 0XD3440000},
	{"V_RNDNE_F32",					0x7E004600,	VOP1, VOP3a, 0XD3460000},
	{"V_FLOOR_F32",					0x7E004800,	VOP1, VOP3a, 0XD3480000},
	{"V_EXP_F32",					0x7E004A00,	VOP1, VOP3a, 0XD34A0000},
	{"V_LOG_CLAMP_F32",				0x7E004C00,	VOP1, VOP3a, 0XD34C0000},
	{"V_LOG_F32",					0x7E004E00,	VOP1, VOP3a, 0XD34E0000},
	{"V_RCP_CLAMP_F32",				0x7E005000,	VOP1, VOP3a, 0XD3500000},
	{"V_RCP_LEGACY_F32",			0x7E005200,	VOP1, VOP3a, 0XD3520000},
	{"V_RCP_F32",					0x7E005400,	VOP1, VOP3a, 0XD3540000},
	{"V_RCP_IFLAG_F32",				0x7E005600,	VOP1, VOP3a, 0XD3560000},
	{"V_RSQ_CLAMP_F32",				0x7E005800,	VOP1, VOP3a, 0XD3580000},
	{"V_RSQ_LEGACY_F32",			0x7E005A00,	VOP1, VOP3a, 0XD35A0000},
	{"V_RSQ_F32",					0x7E005C00,	VOP1, VOP3a, 0XD35C0000},
	{"V_RCP_F64",					0x7E005E00,	VOP1, VOP3a, 0XD35E0000},
	{"V_RCP_CLAMP_F64",				0x7E006000,	VOP1, VOP3a, 0XD3600000},
	{"V_RSQ_F64",					0x7E006200,	VOP1, VOP3a, 0XD3620000},
	{"V_RSQ_CLAMP_F64",				0x7E006400,	VOP1, VOP3a, 0XD3640000},
	{"V_SQRT_F32",					0x7E006600,	VOP1, VOP3a, 0XD3660000},
	{"V_SQRT_F64",					0x7E006800,	VOP1, VOP3a, 0XD3680000},
	{"V_SIN_F32",					0x7E006A00,	VOP1, VOP3a, 0XD36A0000},
	{"V_COS_F32",					0x7E006C00,	VOP1, VOP3a, 0XD36C0000},
	{"V_NOT_B32",					0x7E006E00,	VOP1, VOP3a, 0XD36E0000},
	{"V_BFREV_B32",					0x7E007000,	VOP1, VOP3a, 0XD3700000},
	{"V_FFBH_U32",					0x7E007200,	VOP1, VOP3a, 0XD3720000},
	{"V_FFBL_B32",					0x7E007400,	VOP1, VOP3a, 0XD3740000},
	{"V_FFBH_I32",					0x7E007600,	VOP1, VOP3a, 0XD3760000},
	{"V_FREXP_EXP_I32_F64",			0x7E007800,	VOP1, VOP3a, 0XD3780000},
	{"V_FREXP_MANT_F64",			0x7E007A00,	VOP1, VOP3a, 0XD37A0000},
	{"V_FRACT_F64",					0x7E007C00,	VOP1, VOP3a, 0XD37C0000},
	{"V_FREXP_EXP_I32_F32",			0x7E007E00,	VOP1, VOP3a, 0XD37E0000},
	{"V_FREXP_MANT_F32",			0x7E008000,	VOP1, VOP3a, 0XD3800000},
	{"V_CLREXCP",					0x7E008200,	VOP1, VOP3a, 0XD3820000},
	{"V_MOVRELD_B32",				0x7E008400,	VOP1, VOP3a, 0XD3840000},
	{"V_MOVRELS_B32",				0x7E008600,	VOP1, VOP3a, 0XD3860000},
	{"V_MOVRELSD_B32",				0x7E008800,	VOP1, VOP3a, 0XD3880000},

	// VOPC instructions
	{"V_CMP_F_F32",					0x7C000000,	VOPC, VOP3b, 0xD0000000},
	{"V_CMP_LT_F32",				0x7C020000,	VOPC, VOP3b, 0xD0020000},
	{"V_CMP_EQ_F32",				0x7C040000,	VOPC, VOP3b, 0xD0040000},
	{"V_CMP_LE_F32",				0x7C060000,	VOPC, VOP3b, 0xD0060000},
	{"V_CMP_GT_F32",				0x7C080000,	VOPC, VOP3b, 0xD0080000},
	{"V_CMP_LG_F32",				0x7C0A0000,	VOPC, VOP3b, 0xD00A0000},
	{"V_CMP_GE_F32",				0x7C0C0000,	VOPC, VOP3b, 0xD00C0000},
	{"V_CMP_O_F32",					0x7C0E0000,	VOPC, VOP3b, 0xD00E0000},
	{"V_CMP_U_F32",					0x7C100000,	VOPC, VOP3b, 0xD0100000},
	{"V_CMP_NGE_F32",				0x7C120000,	VOPC, VOP3b, 0xD0120000},
	{"V_CMP_NLG_F32",				0x7C140000,	VOPC, VOP3b, 0xD0140000},
	{"V_CMP_NGT_F32",				0x7C160000,	VOPC, VOP3b, 0xD0160000},
	{"V_CMP_NLE_F32",				0x7C180000,	VOPC, VOP3b, 0xD0180000},
	{"V_CMP_NEQ_F32",				0x7C1A0000,	VOPC, VOP3b, 0xD01A0000},
	{"V_CMP_NLT_F32",				0x7C1C0000,	VOPC, VOP3b, 0xD01C0000},
	{"V_CMP_TRU_F32",				0x7C1E0000,	VOPC, VOP3b, 0xD01E0000},
	{"V_CMPX_F_F32",				0x7C200000,	VOPC, VOP3b, 0xD0200000},
	{"V_CMPX_LT_F32",				0x7C220000,	VOPC, VOP3b, 0xD0220000},
	{"V_CMPX_EQ_F32",				0x7C240000,	VOPC, VOP3b, 0xD0240000},
	{"V_CMPX_LE_F32",				0x7C260000,	VOPC, VOP3b, 0xD0260000},
	{"V_CMPX_GT_F32",				0x7C280000,	VOPC, VOP3b, 0xD0280000},
	{"V_CMPX_LG_F32",				0x7C2A0000,	VOPC, VOP3b, 0xD02A0000},
	{"V_CMPX_GE_F32",				0x7C2C0000,	VOPC, VOP3b, 0xD02C0000},
	{"V_CMPX_O_F32",				0x7C2E0000,	VOPC, VOP3b, 0xD02E0000},
	{"V_CMPX_U_F32",				0x7C300000,	VOPC, VOP3b, 0xD0300000},
	{"V_CMPX_NGE_F32",				0x7C320000,	VOPC, VOP3b, 0xD0320000},
	{"V_CMPX_NLG_F32",				0x7C340000,	VOPC, VOP3b, 0xD0340000},
	{"V_CMPX_NGT_F32",				0x7C360000,	VOPC, VOP3b, 0xD0360000},
	{"V_CMPX_NLE_F32",				0x7C380000,	VOPC, VOP3b, 0xD0380000},
	{"V_CMPX_NEQ_F32",				0x7C3A0000,	VOPC, VOP3b, 0xD03A0000},
	{"V_CMPX_NLT_F32",				0x7C3C0000,	VOPC, VOP3b, 0xD03C0000},
	{"V_CMPX_TRU_F32",				0x7C3E0000,	VOPC, VOP3b, 0xD03E0000},
	{"V_CMP_F_F64",					0x7C400000,	VOPC, VOP3b, 0xD0400000},
	{"V_CMP_LT_F64",				0x7C420000,	VOPC, VOP3b, 0xD0420000},
	{"V_CMP_EQ_F64",				0x7C440000,	VOPC, VOP3b, 0xD0440000},
	{"V_CMP_LE_F64",				0x7C460000,	VOPC, VOP3b, 0xD0460000},
	{"V_CMP_GT_F64",				0x7C480000,	VOPC, VOP3b, 0xD0480000},
	{"V_CMP_LG_F64",				0x7C4A0000,	VOPC, VOP3b, 0xD04A0000},
	{"V_CMP_GE_F64",				0x7C4C0000,	VOPC, VOP3b, 0xD04C0000},
	{"V_CMP_O_F64",					0x7C4E0000,	VOPC, VOP3b, 0xD04E0000},
	{"V_CMP_U_F64",					0x7C500000,	VOPC, VOP3b, 0xD0500000},
	{"V_CMP_NGE_F64",				0x7C520000,	VOPC, VOP3b, 0xD0520000},
	{"V_CMP_NLG_F64",				0x7C540000,	VOPC, VOP3b, 0xD0540000},
	{"V_CMP_NGT_F64",				0x7C560000,	VOPC, VOP3b, 0xD0560000},
	{"V_CMP_NLE_F64",				0x7C580000,	VOPC, VOP3b, 0xD0580000},
	{"V_CMP_NEQ_F64",				0x7C5A0000,	VOPC, VOP3b, 0xD05A0000},
	{"V_CMP_NLT_F64",				0x7C5C0000,	VOPC, VOP3b, 0xD05C0000},
	{"V_CMP_TRU_F64",				0x7C5E0000,	VOPC, VOP3b, 0xD05E0000},
	{"V_CMPX_F_F64",				0x7C600000,	VOPC, VOP3b, 0xD0600000},
	{"V_CMPX_LT_F64",				0x7C620000,	VOPC, VOP3b, 0xD0620000},
	{"V_CMPX_EQ_F64",				0x7C640000,	VOPC, VOP3b, 0xD0640000},
	{"V_CMPX_LE_F64",				0x7C660000,	VOPC, VOP3b, 0xD0660000},
	{"V_CMPX_GT_F64",				0x7C680000,	VOPC, VOP3b, 0xD0680000},
	{"V_CMPX_LG_F64",				0x7C6A0000,	VOPC, VOP3b, 0xD06A0000},
	{"V_CMPX_GE_F64",				0x7C6C0000,	VOPC, VOP3b, 0xD06C0000},
	{"V_CMPX_O_F64",				0x7C6E0000,	VOPC, VOP3b, 0xD06E0000},
	{"V_CMPX_U_F64",				0x7C700000,	VOPC, VOP3b, 0xD0700000},
	{"V_CMPX_NGE_F64",				0x7C720000,	VOPC, VOP3b, 0xD0720000},
	{"V_CMPX_NLG_F64",				0x7C740000,	VOPC, VOP3b, 0xD0740000},
	{"V_CMPX_NGT_F64",				0x7C760000,	VOPC, VOP3b, 0xD0760000},
	{"V_CMPX_NLE_F64",				0x7C780000,	VOPC, VOP3b, 0xD0780000},
	{"V_CMPX_NEQ_F64",				0x7C7A0000,	VOPC, VOP3b, 0xD07A0000},
	{"V_CMPX_NLT_F64",				0x7C7C0000,	VOPC, VOP3b, 0xD07C0000},
	{"V_CMPX_TRU_F64",				0x7C7E0000,	VOPC, VOP3b, 0xD07E0000},
	{"V_CMPS_F_F32",				0x7C800000,	VOPC, VOP3b, 0xD0800000},
	{"V_CMPS_LT_F32",				0x7C820000,	VOPC, VOP3b, 0xD0820000},
	{"V_CMPS_EQ_F32",				0x7C840000,	VOPC, VOP3b, 0xD0840000},
	{"V_CMPS_LE_F32",				0x7C860000,	VOPC, VOP3b, 0xD0860000},
	{"V_CMPS_GT_F32",				0x7C880000,	VOPC, VOP3b, 0xD0880000},
	{"V_CMPS_LG_F32",				0x7C8A0000,	VOPC, VOP3b, 0xD08A0000},
	{"V_CMPS_GE_F32",				0x7C8C0000,	VOPC, VOP3b, 0xD08C0000},
	{"V_CMPS_O_F32",				0x7C8E0000,	VOPC, VOP3b, 0xD08E0000},
	{"V_CMPS_U_F32",				0x7C900000,	VOPC, VOP3b, 0xD0900000},
	{"V_CMPS_NGE_F32",				0x7C920000,	VOPC, VOP3b, 0xD0920000},
	{"V_CMPS_NLG_F32",				0x7C940000,	VOPC, VOP3b, 0xD0940000},
	{"V_CMPS_NGT_F32",				0x7C960000,	VOPC, VOP3b, 0xD0960000},
	{"V_CMPS_NLE_F32",				0x7C980000,	VOPC, VOP3b, 0xD0980000},
	{"V_CMPS_NEQ_F32",				0x7C9A0000,	VOPC, VOP3b, 0xD09A0000},
	{"V_CMPS_NLT_F32",				0x7C9C0000,	VOPC, VOP3b, 0xD09C0000},
	{"V_CMPS_TRU_F32",				0x7C9E0000,	VOPC, VOP3b, 0xD09E0000},
	{"V_CMPSX_F_F32",				0x7CA00000,	VOPC, VOP3b, 0xD0A00000},
	{"V_CMPSX_LT_F32",				0x7CA20000,	VOPC, VOP3b, 0xD0A20000},
	{"V_CMPSX_EQ_F32",				0x7CA40000,	VOPC, VOP3b, 0xD0A40000},
	{"V_CMPSX_LE_F32",				0x7CA60000,	VOPC, VOP3b, 0xD0A60000},
	{"V_CMPSX_GT_F32",				0x7CA80000,	VOPC, VOP3b, 0xD0A80000},
	{"V_CMPSX_LG_F32",				0x7CAA0000,	VOPC, VOP3b, 0xD0AA0000},
	{"V_CMPSX_GE_F32",				0x7CAC0000,	VOPC, VOP3b, 0xD0AC0000},
	{"V_CMPSX_O_F32",				0x7CAE0000,	VOPC, VOP3b, 0xD0AE0000},
	{"V_CMPSX_U_F32",				0x7CB00000,	VOPC, VOP3b, 0xD0B00000},
	{"V_CMPSX_NGE_F32",				0x7CB20000,	VOPC, VOP3b, 0xD0B20000},
	{"V_CMPSX_NLG_F32",				0x7CB40000,	VOPC, VOP3b, 0xD0B40000},
	{"V_CMPSX_NGT_F32",				0x7CB60000,	VOPC, VOP3b, 0xD0B60000},
	{"V_CMPSX_NLE_F32",				0x7CB80000,	VOPC, VOP3b, 0xD0B80000},
	{"V_CMPSX_NEQ_F32",				0x7CBA0000,	VOPC, VOP3b, 0xD0BA0000},
	{"V_CMPSX_NLT_F32",				0x7CBC0000,	VOPC, VOP3b, 0xD0BC0000},
	{"V_CMPSX_TRU_F32",				0x7CBE0000,	VOPC, VOP3b, 0xD0BE0000},
	{"V_CMPS_F_F64",				0x7CC00000,	VOPC, VOP3b, 0xD0C00000},
	{"V_CMPS_LT_F64",				0x7CC20000,	VOPC, VOP3b, 0xD0C20000},
	{"V_CMPS_EQ_F64",				0x7CC40000,	VOPC, VOP3b, 0xD0C40000},
	{"V_CMPS_LE_F64",				0x7CC60000,	VOPC, VOP3b, 0xD0C60000},
	{"V_CMPS_GT_F64",				0x7CC80000,	VOPC, VOP3b, 0xD0C80000},
	{"V_CMPS_LG_F64",				0x7CCA0000,	VOPC, VOP3b, 0xD0CA0000},
	{"V_CMPS_GE_F64",				0x7CCC0000,	VOPC, VOP3b, 0xD0CC0000},
	{"V_CMPS_O_F64",				0x7CCE0000,	VOPC, VOP3b, 0xD0CE0000},
	{"V_CMPS_U_F64",				0x7CD00000,	VOPC, VOP3b, 0xD0D00000},
	{"V_CMPS_NGE_F64",				0x7CD20000,	VOPC, VOP3b, 0xD0D20000},
	{"V_CMPS_NLG_F64",				0x7CD40000,	VOPC, VOP3b, 0xD0D40000},
	{"V_CMPS_NGT_F64",				0x7CD60000,	VOPC, VOP3b, 0xD0D60000},
	{"V_CMPS_NLE_F64",				0x7CD80000,	VOPC, VOP3b, 0xD0D80000},
	{"V_CMPS_NEQ_F64",				0x7CDA0000,	VOPC, VOP3b, 0xD0DA0000},
	{"V_CMPS_NLT_F64",				0x7CDC0000,	VOPC, VOP3b, 0xD0DC0000},
	{"V_CMPS_TRU_F64",				0x7CDE0000,	VOPC, VOP3b, 0xD0DE0000},
	{"V_CMPSX_F_F64",				0x7CE00000,	VOPC, VOP3b, 0xD0E00000},
	{"V_CMPSX_LT_F64",				0x7CE20000,	VOPC, VOP3b, 0xD0E20000},
	{"V_CMPSX_EQ_F64",				0x7CE40000,	VOPC, VOP3b, 0xD0E40000},
	{"V_CMPSX_LE_F64",				0x7CE60000,	VOPC, VOP3b, 0xD0E60000},
	{"V_CMPSX_GT_F64",				0x7CE80000,	VOPC, VOP3b, 0xD0E80000},
	{"V_CMPSX_LG_F64",				0x7CEA0000,	VOPC, VOP3b, 0xD0EA0000},
	{"V_CMPSX_GE_F64",				0x7CEC0000,	VOPC, VOP3b, 0xD0EC0000},
	{"V_CMPSX_O_F64",				0x7CEE0000,	VOPC, VOP3b, 0xD0EE0000},
	{"V_CMPSX_U_F64",				0x7CF00000,	VOPC, VOP3b, 0xD0F00000},
	{"V_CMPSX_NGE_F64",				0x7CF20000,	VOPC, VOP3b, 0xD0F20000},
	{"V_CMPSX_NLG_F64",				0x7CF40000,	VOPC, VOP3b, 0xD0F40000},
	{"V_CMPSX_NGT_F64",				0x7CF60000,	VOPC, VOP3b, 0xD0F60000},
	{"V_CMPSX_NLE_F64",				0x7CF80000,	VOPC, VOP3b, 0xD0F80000},
	{"V_CMPSX_NEQ_F64",				0x7CFA0000,	VOPC, VOP3b, 0xD0FA0000},
	{"V_CMPSX_NLT_F64",				0x7CFC0000,	VOPC, VOP3b, 0xD0FC0000},
	{"V_CMPSX_TRU_F64",				0x7CFE0000,	VOPC, VOP3b, 0xD0FE0000},
	{"V_CMP_F_I32",					0x7D000000,	VOPC, VOP3b, 0xD1000000},
	{"V_CMP_LT_I32",				0x7D020000,	VOPC, VOP3b, 0xD1020000},
	{"V_CMP_EQ_I32",				0x7D040000,	VOPC, VOP3b, 0xD1040000},
	{"V_CMP_LE_I32",				0x7D060000,	VOPC, VOP3b, 0xD1060000},
	{"V_CMP_GT_I32",				0x7D080000,	VOPC, VOP3b, 0xD1080000},
	{"V_CMP_LG_I32",				0x7D0A0000,	VOPC, VOP3b, 0xD10A0000},
	{"V_CMP_GE_I32",				0x7D0C0000,	VOPC, VOP3b, 0xD10C0000},
	{"V_CMP_TRU_I32",				0x7D0E0000,	VOPC, VOP3b, 0xD10E0000},
	{"V_CMPX_F_I32",				0x7D200000,	VOPC, VOP3b, 0xD1200000},
	{"V_CMPX_LT_I32",				0x7D220000,	VOPC, VOP3b, 0xD1220000},
	{"V_CMPX_EQ_I32",				0x7D240000,	VOPC, VOP3b, 0xD1240000},
	{"V_CMPX_LE_I32",				0x7D260000,	VOPC, VOP3b, 0xD1260000},
	{"V_CMPX_GT_I32",				0x7D280000,	VOPC, VOP3b, 0xD1280000},
	{"V_CMPX_LG_I32",				0x7D2A0000,	VOPC, VOP3b, 0xD12A0000},
	{"V_CMPX_GE_I32",				0x7D2C0000,	VOPC, VOP3b, 0xD12C0000},
	{"V_CMPX_TRU_I32",				0x7D2E0000,	VOPC, VOP3b, 0xD12E0000},
	{"V_CMP_F_I64",					0x7D400000,	VOPC, VOP3b, 0xD1400000},
	{"V_CMP_LT_I64",				0x7D420000,	VOPC, VOP3b, 0xD1420000},
	{"V_CMP_EQ_I64",				0x7D440000,	VOPC, VOP3b, 0xD1440000},
	{"V_CMP_LE_I64",				0x7D460000,	VOPC, VOP3b, 0xD1460000},
	{"V_CMP_GT_I64",				0x7D480000,	VOPC, VOP3b, 0xD1480000},
	{"V_CMP_LG_I64",				0x7D4A0000,	VOPC, VOP3b, 0xD14A0000},
	{"V_CMP_GE_I64",				0x7D4C0000,	VOPC, VOP3b, 0xD14C0000},
	{"V_CMP_TRU_I64",				0x7D4E0000,	VOPC, VOP3b, 0xD14E0000},
	{"V_CMPX_F_I64",				0x7D600000,	VOPC, VOP3b, 0xD1600000},
	{"V_CMPX_LT_I64",				0x7D620000,	VOPC, VOP3b, 0xD1620000},
	{"V_CMPX_EQ_I64",				0x7D640000,	VOPC, VOP3b, 0xD1640000},
	{"V_CMPX_LE_I64",				0x7D660000,	VOPC, VOP3b, 0xD1660000},
	{"V_CMPX_GT_I64",				0x7D680000,	VOPC, VOP3b, 0xD1680000},
	{"V_CMPX_LG_I64",				0x7D6A0000,	VOPC, VOP3b, 0xD16A0000},
	{"V_CMPX_GE_I64",				0x7D6C0000,	VOPC, VOP3b, 0xD16C0000},
	{"V_CMPX_TRU_I64",				0x7D6E0000,	VOPC, VOP3b, 0xD16E0000},
	{"V_CMP_F_U32",					0x7D800000,	VOPC, VOP3b, 0xD1800000},
	{"V_CMP_LT_U32",				0x7D820000,	VOPC, VOP3b, 0xD1820000},
	{"V_CMP_EQ_U32",				0x7D840000,	VOPC, VOP3b, 0xD1840000},
	{"V_CMP_LE_U32",				0x7D860000,	VOPC, VOP3b, 0xD1860000},
	{"V_CMP_GT_U32",				0x7D880000,	VOPC, VOP3b, 0xD1880000},
	{"V_CMP_LG_U32",				0x7D8A0000,	VOPC, VOP3b, 0xD18A0000},
	{"V_CMP_GE_U32",				0x7D8C0000,	VOPC, VOP3b, 0xD18C0000},
	{"V_CMP_TRU_U32",				0x7D8E0000,	VOPC, VOP3b, 0xD18E0000},
	{"V_CMPX_F_U32",				0x7DA00000,	VOPC, VOP3b, 0xD1A00000},
	{"V_CMPX_LT_U32",				0x7DA20000,	VOPC, VOP3b, 0xD1A20000},
	{"V_CMPX_EQ_U32",				0x7DA40000,	VOPC, VOP3b, 0xD1A40000},
	{"V_CMPX_LE_U32",				0x7DA60000,	VOPC, VOP3b, 0xD1A60000},
	{"V_CMPX_GT_U32",				0x7DA80000,	VOPC, VOP3b, 0xD1A80000},
	{"V_CMPX_LG_U32",				0x7DAA0000,	VOPC, VOP3b, 0xD1AA0000},
	{"V_CMPX_GE_U32",				0x7DAC0000,	VOPC, VOP3b, 0xD1AC0000},
	{"V_CMPX_TRU_U32",				0x7DAE0000,	VOPC, VOP3b, 0xD1AE0000},
	{"V_CMP_F_U64",					0x7DC00000,	VOPC, VOP3b, 0xD1C00000},
	{"V_CMP_LT_U64",				0x7DC20000,	VOPC, VOP3b, 0xD1C20000},
	{"V_CMP_EQ_U64",				0x7DC40000,	VOPC, VOP3b, 0xD1C40000},
	{"V_CMP_LE_U64",				0x7DC60000,	VOPC, VOP3b, 0xD1C60000},
	{"V_CMP_GT_U64",				0x7DC80000,	VOPC, VOP3b, 0xD1C80000},
	{"V_CMP_LG_U64",				0x7DCA0000,	VOPC, VOP3b, 0xD1CA0000},
	{"V_CMP_GE_U64",				0x7DCC0000,	VOPC, VOP3b, 0xD1CC0000},
	{"V_CMP_TRU_U64",				0x7DCE0000,	VOPC, VOP3b, 0xD1CE0000},
	{"V_CMPX_F_U64",				0x7DE00000,	VOPC, VOP3b, 0xD1E00000},
	{"V_CMPX_LT_U64",				0x7DE20000,	VOPC, VOP3b, 0xD1E20000},
	{"V_CMPX_EQ_U64",				0x7DE40000,	VOPC, VOP3b, 0xD1E40000},
	{"V_CMPX_LE_U64",				0x7DE60000,	VOPC, VOP3b, 0xD1E60000},
	{"V_CMPX_GT_U64",				0x7DE80000,	VOPC, VOP3b, 0xD1E80000},
	{"V_CMPX_LG_U64",				0x7DEA0000,	VOPC, VOP3b, 0xD1EA0000},
	{"V_CMPX_GE_U64",				0x7DEC0000,	VOPC, VOP3b, 0xD1EC0000},
	{"V_CMPX_TRU_U64",				0x7DEE0000,	VOPC, VOP3b, 0xD1EE0000},
	{"V_CMP_CLASS_F32",				0x7D100000,	VOPC, VOP3b, 0xD1100000},
	{"V_CMPX_CLASS_F32",			0x7D300000,	VOPC, VOP3b, 0xD1300000},
	{"V_CMP_CLASS_F64",				0x7D500000,	VOPC, VOP3b, 0xD1500000},
	{"V_CMPX_CLASS_F64",			0x7D700000,	VOPC, VOP3b, 0xD1700000},

	// VOP3a instructions
	{"V_MAD_LEGACY_F32",			0xD2800000, VOP3a, NONE, 0},
	{"V_MAD_F32",					0xD2820000, VOP3a, NONE, 0},
	{"V_MAD_I32_I24",				0xD2840000, VOP3a, NONE, 0},
	{"V_MAD_U32_U24",				0xD2860000, VOP3a, NONE, 0},
	{"V_CUBEID_F32",				0xD2880000, VOP3a, NONE, 0},
	{"V_CUBESC_F32",				0xD28A0000, VOP3a, NONE, 0},
	{"V_CUBETC_F32",				0xD28C0000, VOP3a, NONE, 0},
	{"V_CUBEMA_F32",				0xD28E0000, VOP3a, NONE, 0},
	{"V_BFE_U32",					0xD2900000, VOP3a, NONE, 0},
	{"V_BFE_I32",					0xD2920000, VOP3a, NONE, 0},
	{"V_BFI_B32",					0xD2940000, VOP3a, NONE, 0},
	{"V_FMA_F32",					0xD2960000, VOP3a, NONE, 0},
	{"V_FMA_F64",					0xD2980000, VOP3a, NONE, 0},
	{"V_LERP_U8",					0xD29A0000, VOP3a, NONE, 0},
	{"V_ALIGNBIT_B32",				0xD29C0000, VOP3a, NONE, 0},
	{"V_ALIGNBYTE_B32",				0xD29E0000, VOP3a, NONE, 0},
	{"V_MULLIT_F32",				0xD2A00000, VOP3a, NONE, 0},
	{"V_MIN3_F32",					0xD2A20000, VOP3a, NONE, 0},
	{"V_MIN3_I32",					0xD2A40000, VOP3a, NONE, 0},
	{"V_MIN3_U32",					0xD2A60000, VOP3a, NONE, 0},
	{"V_MAX3_F32",					0xD2A80000, VOP3a, NONE, 0},
	{"V_MAX3_I32",					0xD2AA0000, VOP3a, NONE, 0},
	{"V_MAX3_U32",					0xD2AC0000, VOP3a, NONE, 0},
	{"V_MED3_F32",					0xD2AE0000, VOP3a, NONE, 0},
	{"V_MED3_I32",					0xD2B00000, VOP3a, NONE, 0},
	{"V_MED3_U32",					0xD2B20000, VOP3a, NONE, 0},
	{"V_SAD_U8",					0xD2B40000, VOP3a, NONE, 0},
	{"V_SAD_HI_U8",					0xD2B60000, VOP3a, NONE, 0},
	{"V_SAD_U16",					0xD2B80000, VOP3a, NONE, 0},
	{"V_SAD_U32",					0xD2BA0000, VOP3a, NONE, 0},
	{"V_CVT_PK_U8_F32",				0xD2BC0000, VOP3a, NONE, 0},
	{"V_DIV_FIXUP_F32",				0xD2BE0000, VOP3a, NONE, 0},
	{"V_DIV_FIXUP_F64",				0xD2C00000, VOP3a, NONE, 0},
	{"V_LSHL_B64",					0xD2C20000, VOP3a, NONE, 0},
	{"V_LSHR_B64",					0xD2C40000, VOP3a, NONE, 0},
	{"V_ASHR_I64",					0xD2C60000, VOP3a, NONE, 0},
	{"V_ADD_F64",					0xD2C80000, VOP3a, NONE, 0},
	{"V_MUL_F64",					0xD2CA0000, VOP3a, NONE, 0},
	{"V_MIN_F64",					0xD2CC0000, VOP3a, NONE, 0},
	{"V_MAX_F64",					0xD2CE0000, VOP3a, NONE, 0},
	{"V_LDEXP_F64",					0xD2D00000, VOP3a, NONE, 0},
	{"V_MUL_LO_U32",				0xD2D20000, VOP3a, NONE, 0},
	{"V_MUL_HI_U32",				0xD2D40000, VOP3a, NONE, 0},
	{"V_MUL_LO_I32",				0xD2D60000, VOP3a, NONE, 0},
	{"V_MUL_HI_I32",				0xD2D80000, VOP3a, NONE, 0},
	{"V_DIV_FMAS_F32",				0xD2DE0000, VOP3a, NONE, 0},
	{"V_DIV_FMAS_F64",				0xD2E00000, VOP3a, NONE, 0},
	{"V_MSAD_U8",					0xD2E20000, VOP3a, NONE, 0},
	{"V_QSAD_U8",					0xD2E40000, VOP3a, NONE, 0},
	{"V_MQSAD_U8",					0xD2E60000, VOP3a, NONE, 0},
	{"V_TRIG_PREOP_F64",			0xD2E80000, VOP3a, NONE, 0},

	// VOP3b instructions
	{"V_DIV_SCALE_F32",				0xD2DA0000, VOP3b, NONE, 0},
	{"V_DIV_SCALE_F64",				0xD2DC0000, VOP3b, NONE, 0},

	// VINTRP instructions
	// {"V_INTERP_P1_F32",			0xC8000000, VINTRP},
	// {"V_INTERP_P2_F32",			0xC8010000, VINTRP},
	// {"V_INTERP_MOV_F32",			0xC8020000, VINTRP},

	// DS instructions
	{"DS_ADD_U32",					0xD8000000, DS, NONE, 0},
	{"DS_SUB_U32",					0xD8040000, DS, NONE, 0},
	{"DS_RSUB_U32",					0xD8080000, DS, NONE, 0},
	{"DS_INC_U32",					0xD80C0000, DS, NONE, 0},
	{"DS_DEC_U32",					0xD8100000, DS, NONE, 0},
	{"DS_MIN_I32",					0xD8140000, DS, NONE, 0},
	{"DS_MAX_I32",					0xD8180000, DS, NONE, 0},
	{"DS_MIN_U32",					0xD81C0000, DS, NONE, 0},
	{"DS_MAX_U32",					0xD8200000, DS, NONE, 0},
	{"DS_AND_B32",					0xD8240000, DS, NONE, 0},
	{"DS_OR_B32",					0xD8280000, DS, NONE, 0},
	{"DS_XOR_B32",					0xD82C0000, DS, NONE, 0},
	{"DS_MSKOR_B32",				0xD8300000, DS, NONE, 0},
	{"DS_WRITE_B32",				0xD8340000, DS, NONE, 0},
	{"DS_WRITE2_B32",				0xD8380000, DS, NONE, 0},
	{"DS_WRITE2ST64_B32",			0xD83C0000, DS, NONE, 0},
	{"DS_CMPST_B32",				0xD8400000, DS, NONE, 0},
	{"DS_CMPST_F32",				0xD8440000, DS, NONE, 0},
	{"DS_MIN_F32",					0xD8480000, DS, NONE, 0},
	{"DS_MAX_F32",					0xD84C0000, DS, NONE, 0},
	{"DS_GWS_INIT",					0xD8640000, DS, NONE, 0},
	{"DS_GWS_SEMA_V",				0xD8680000, DS, NONE, 0},
	{"DS_GWS_SEMA_BR",				0xD86C0000, DS, NONE, 0},
	{"DS_GWS_SEMA_P",				0xD8700000, DS, NONE, 0},
	{"DS_GWS_BARRIER",				0xD8740000, DS, NONE, 0},
	{"DS_WRITE_B8",					0xD8780000, DS, NONE, 0},
	{"DS_WRITE_B16",				0xD87C0000, DS, NONE, 0},
	{"DS_ADD_RTN_U32",				0xD8800000, DS, NONE, 0},
	{"DS_SUB_RTN_U32",				0xD8840000, DS, NONE, 0},
	{"DS_RSUB_RTN_U32",				0xD8880000, DS, NONE, 0},
	{"DS_INC_RTN_U32",				0xD88C0000, DS, NONE, 0},
	{"DS_DEC_RTN_U32",				0xD8900000, DS, NONE, 0},
	{"DS_MIN_RTN_I32",				0xD8940000, DS, NONE, 0},
	{"DS_MAX_RTN_I32",				0xD8980000, DS, NONE, 0},
	{"DS_MIN_RTN_U32",				0xD89C0000, DS, NONE, 0},
	{"DS_MAX_RTN_U32",				0xD8A00000, DS, NONE, 0},
	{"DS_AND_RTN_B32",				0xD8A40000, DS, NONE, 0},
	{"DS_OR_RTN_B32",				0xD8A80000, DS, NONE, 0},
	{"DS_XOR_RTN_B32",				0xD8AC0000, DS, NONE, 0},
	{"DS_MSKOR_RTN_B32",			0xD8B00000, DS, NONE, 0},
	{"DS_WRXCHG_RTN_B32",			0xD8B40000, DS, NONE, 0},
	{"DS_WRXCHG2_RTN_B32",			0xD8B80000, DS, NONE, 0},
	{"DS_WRXCHG2ST64_RTN_B32",		0xD8BC0000, DS, NONE, 0},
	{"DS_CMPST_RTN_B32",			0xD8C00000, DS, NONE, 0},
	{"DS_CMPST_RTN_F32",			0xD8C40000, DS, NONE, 0},
	{"DS_MIN_RTN_F32",				0xD8C80000, DS, NONE, 0},
	{"DS_MAX_RTN_F32",				0xD8CC0000, DS, NONE, 0},
	{"DS_SWIZZLE_B32",				0xD8D40000, DS, NONE, 0},
	{"DS_READ_B32",					0xD8D80000, DS, NONE, 0},
	{"DS_READ2_B32",				0xD8DC0000, DS, NONE, 0},
	{"DS_READ2ST64_B32",			0xD8E00000, DS, NONE, 0},
	{"DS_READ_I8",					0xD8E40000, DS, NONE, 0},
	{"DS_READ_U8",					0xD8E80000, DS, NONE, 0},
	{"DS_READ_I16",					0xD8EC0000, DS, NONE, 0},
	{"DS_READ_U16",					0xD8F00000, DS, NONE, 0},
	{"DS_CONSUME",					0xD8F40000, DS, NONE, 0},
	{"DS_APPEND",					0xD8F80000, DS, NONE, 0},
	{"DS_ORDERED_COUNT",			0xD8FC0000, DS, NONE, 0},
	{"DS_ADD_U64",					0xD9000000, DS, NONE, 0},
	{"DS_SUB_U64",					0xD9040000, DS, NONE, 0},
	{"DS_RSUB_U64",					0xD9080000, DS, NONE, 0},
	{"DS_INC_U64",					0xD90C0000, DS, NONE, 0},
	{"DS_DEC_U64",					0xD9100000, DS, NONE, 0},
	{"DS_MIN_I64",					0xD9140000, DS, NONE, 0},
	{"DS_MAX_I64",					0xD9180000, DS, NONE, 0},
	{"DS_MIN_U64",					0xD91C0000, DS, NONE, 0},
	{"DS_MAX_U64",					0xD9200000, DS, NONE, 0},
	{"DS_AND_B64",					0xD9240000, DS, NONE, 0},
	{"DS_OR_B64",					0xD9280000, DS, NONE, 0},
	{"DS_XOR_B64",					0xD92C0000, DS, NONE, 0},
	{"DS_MSKOR_B64",				0xD9300000, DS, NONE, 0},
	{"DS_WRITE_B64",				0xD9340000, DS, NONE, 0},
	{"DS_WRITE2_B64",				0xD9380000, DS, NONE, 0},
	{"DS_WRITE2ST64_B64",			0xD93C0000, DS, NONE, 0},
	{"DS_CMPST_B64",				0xD9400000, DS, NONE, 0},
	{"DS_CMPST_F64",				0xD9440000, DS, NONE, 0},
	{"DS_MIN_F64",					0xD9480000, DS, NONE, 0},
	{"DS_MAX_F64",					0xD94C0000, DS, NONE, 0},
	{"DS_ADD_RTN_U64",				0xD9800000, DS, NONE, 0},
	{"DS_SUB_RTN_U64",				0xD9840000, DS, NONE, 0},
	{"DS_RSUB_RTN_U64",				0xD9880000, DS, NONE, 0},
	{"DS_INC_RTN_U64",				0xD98C0000, DS, NONE, 0},
	{"DS_DEC_RTN_U64",				0xD9900000, DS, NONE, 0},
	{"DS_MIN_RTN_I64",				0xD9940000, DS, NONE, 0},
	{"DS_MAX_RTN_I64",				0xD9980000, DS, NONE, 0},
	{"DS_MIN_RTN_U64",				0xD99C0000, DS, NONE, 0},
	{"DS_MAX_RTN_U64",				0xD9A00000, DS, NONE, 0},
	{"DS_AND_RTN_B64",				0xD9A40000, DS, NONE, 0},
	{"DS_OR_RTN_B64",				0xD9A80000, DS, NONE, 0},
	{"DS_XOR_RTN_B64",				0xD9AC0000, DS, NONE, 0},
	{"DS_MSKOR_RTN_B64",			0xD9B00000, DS, NONE, 0},
	{"DS_WRXCHG_RTN_B64",			0xD9B40000, DS, NONE, 0},
	{"DS_WRXCHG2_RTN_B64",			0xD9B80000, DS, NONE, 0},
	{"DS_WRXCHG2ST64_RTN_B64",		0xD9BC0000, DS, NONE, 0},
	{"DS_CMPST_RTN_B64",			0xD9C00000, DS, NONE, 0},
	{"DS_CMPST_RTN_F64",			0xD9C40000, DS, NONE, 0},
	{"DS_MIN_RTN_F64",				0xD9C80000, DS, NONE, 0},
	{"DS_MAX_RTN_F64",				0xD9CC0000, DS, NONE, 0},
	{"DS_READ_B64",					0xD9D80000, DS, NONE, 0},
	{"DS_READ2_B64",				0xD9DC0000, DS, NONE, 0},
	{"DS_READ2ST64_B64",			0xD9E00000, DS, NONE, 0},
	{"DS_ADD_SRC2_U32",				0xDA000000, DS, NONE, 0},
	{"DS_SUB_SRC2_U32",				0xDA040000, DS, NONE, 0},
	{"DS_RSUB_SRC2_U32",			0xDA080000, DS, NONE, 0},
	{"DS_INC_SRC2_U32",				0xDA0C0000, DS, NONE, 0},
	{"DS_DEC_SRC2_U32",				0xDA100000, DS, NONE, 0},
	{"DS_MIN_SRC2_I32",				0xDA140000, DS, NONE, 0},
	{"DS_MAX_SRC2_I32",				0xDA180000, DS, NONE, 0},
	{"DS_MIN_SRC2_U32",				0xDA1C0000, DS, NONE, 0},
	{"DS_MAX_SRC2_U32",				0xDA200000, DS, NONE, 0},
	{"DS_AND_SRC2_B32B",			0xDA240000, DS, NONE, 0},
	{"DS_OR_SRC2_B32",				0xDA280000, DS, NONE, 0},
	{"DS_XOR_SRC2_B32",				0xDA2C0000, DS, NONE, 0},
	{"DS_WRITE_SRC2_B32",			0xDA300000, DS, NONE, 0},
	{"DS_MIN_SRC2_F32",				0xDA480000, DS, NONE, 0},
	{"DS_MAX_SRC2_F32",				0xDA4C0000, DS, NONE, 0},
	{"DS_ADD_SRC2_U64",				0xDB000000, DS, NONE, 0},
	{"DS_SUB_SRC2_U64",				0xDB040000, DS, NONE, 0},
	{"DS_RSUB_SRC2_U64",			0xDB080000, DS, NONE, 0},
	{"DS_INC_SRC2_U64",				0xDB0C0000, DS, NONE, 0},
	{"DS_DEC_SRC2_U64",				0xDB100000, DS, NONE, 0},
	{"DS_MIN_SRC2_I64",				0xDB140000, DS, NONE, 0},
	{"DS_MAX_SRC2_I64",				0xDB180000, DS, NONE, 0},
	{"DS_MIN_SRC2_U64",				0xDB1C0000, DS, NONE, 0},
	{"DS_MAX_SRC2_U64",				0xDB200000, DS, NONE, 0},
	{"DS_AND_SRC2_B64",				0xDB240000, DS, NONE, 0},
	{"DS_OR_SRC2_B64",				0xDB280000, DS, NONE, 0},
	{"DS_XOR_SRC2_B64",				0xDB2C0000, DS, NONE, 0},
	{"DS_WRITE_SRC2_B64",			0xDB300000, DS, NONE, 0},
	{"DS_MIN_SRC2_F64",				0xDB480000, DS, NONE, 0},
	{"DS_MAX_SRC2_F64",				0xDB4C0000, DS, NONE, 0},

	// MUBUF instructions
	{"BUFFER_LOAD_FORMAT_X",		0xE0000000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_FORMAT_XY",		0xE0040000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_FORMAT_XYZ",		0xE0080000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_FORMAT_XYZW",		0xE00C0000, MUBUF, NONE, 0},
	{"BUFFER_STORE_FORMAT_X",		0xE0100000, MUBUF, NONE, 0},
	{"BUFFER_STORE_FORMAT_XY",		0xE0140000, MUBUF, NONE, 0},
	{"BUFFER_STORE_FORMAT_XYZ",		0xE0180000, MUBUF, NONE, 0},
	{"BUFFER_STORE_FORMAT_XYZW",	0xE01C0000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_UBYTE",			0xE0200000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_SBYTE",			0xE0240000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_USHORT",			0xE0280000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_SSHORT",			0xE02C0000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_DWORD",			0xE0300000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_DWORDX2",			0xE0340000, MUBUF, NONE, 0},
	{"BUFFER_LOAD_DWORDX4",			0xE0380000, MUBUF, NONE, 0},
	{"BUFFER_STORE_BYTE",			0xE0600000, MUBUF, NONE, 0},
	{"BUFFER_STORE_SHORT",			0xE0680000, MUBUF, NONE, 0},
	{"BUFFER_STORE_DWORD",			0xE0700000, MUBUF, NONE, 0},
	{"BUFFER_STORE_DWORDX2",		0xE0740000, MUBUF, NONE, 0},
	{"BUFFER_STORE_DWORDX4",		0xE0780000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SWAP",			0xE0C00000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_CMPSWAP",		0xE0C40000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_ADD",			0xE0C80000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SUB",			0xE0CC0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_RSUB",			0xE0D00000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SMIN",			0xE0D40000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_UMIN",			0xE0D80000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SMAX",			0xE0DC0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_UMAX",			0xE0E00000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_AND",			0xE0E40000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_OR",			0xE0E80000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_XOR",			0xE0EC0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_INC",			0xE0F00000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_DEC",			0xE0F40000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_FCMPSWAP",		0xE0F80000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_FMIN",			0xE0FC0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_FMAX",			0xE1000000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SWAP_X2",		0xE1400000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_CMPSWAP_X2",	0xE1440000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_ADD_X2",		0xE1480000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SUB_X2",		0xE14C0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_RSUB_X2",		0xE1500000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SMIN_X2",		0xE1540000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_UMIN_X2",		0xE1580000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_SMAX_X2",		0xE15C0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_UMAX_X2",		0xE1600000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_AND_X2",		0xE1640000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_OR_X2",			0xE1680000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_XOR_X2",		0xE16C0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_INC_X2",		0xE1700000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_DEC_X2",		0xE1740000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_FCMPSWAP_X2",	0xE1780000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_FMIN_X2",		0xE17C0000, MUBUF, NONE, 0},
	{"BUFFER_ATOMIC_FMAX_X2",		0xE1800000, MUBUF, NONE, 0},
	{"BUFFER_WBINVL1_SC",			0xE1C00000, MUBUF, NONE, 0},
	{"BUFFER_WBINVL1",				0xE1C40000, MUBUF, NONE, 0},

	// MTBUF instructions
	{"TBUFFER_LOAD_FORMAT_X",		0xE8000000, MTBUF, NONE, 0},
	{"TBUFFER_LOAD_FORMAT_XY",		0xE8010000, MTBUF, NONE, 0},
	{"TBUFFER_LOAD_FORMAT_XYZ",		0xE8020000, MTBUF, NONE, 0},
	{"TBUFFER_LOAD_FORMAT_XYZW",	0xE8030000, MTBUF, NONE, 0},
	{"TBUFFER_STORE_FORMAT_X",		0xE8040000, MTBUF, NONE, 0},
	{"TBUFFER_STORE_FORMAT_XY",		0xE8050000, MTBUF, NONE, 0},
	{"TBUFFER_STORE_FORMAT_XYZ",	0xE8060000, MTBUF, NONE, 0},
	{"TBUFFER_STORE_FORMAT_XYZW",	0xE8070000, MTBUF, NONE, 0},

	// MIMG instructions
	{"IMAGE_LOAD",					0xF0000000, MIMG, NONE, 0},
	{"IMAGE_LOAD_MIP",				0xF0040000, MIMG, NONE, 0},
	{"IMAGE_LOAD_PCK",				0xF0080000, MIMG, NONE, 0},
	{"IMAGE_LOAD_PCK_SGN",			0xF00C0000, MIMG, NONE, 0},
	{"IMAGE_LOAD_MIP_PCK",			0xF0100000, MIMG, NONE, 0},
	{"IMAGE_LOAD_MIP_PCK_SGN",		0xF0140000, MIMG, NONE, 0},
	{"IMAGE_STORE",					0xF0200000, MIMG, NONE, 0},
	{"IMAGE_STORE_MIP",				0xF0240000, MIMG, NONE, 0},
	{"IMAGE_STORE_PCK",				0xF0280000, MIMG, NONE, 0},
	{"IMAGE_STORE_MIP_PCK",			0xF02C0000, MIMG, NONE, 0},
	{"IMAGE_GET_RESINFO",			0xF0380000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_SWAP",			0xF03C0000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_CMPSWAP",		0xF0400000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_ADD",			0xF0440000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_SUB",			0xF0480000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_RSUB",			0xF04C0000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_SMIN",			0xF0500000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_UMIN",			0xF0540000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_SMAX",			0xF0580000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_UMAX",			0xF05C0000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_AND",			0xF0600000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_OR",				0xF0640000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_XOR",			0xF0680000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_INC",			0xF06C0000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_DEC",			0xF0700000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_FCMPSWAP",		0xF0740000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_FMIN",			0xF0780000, MIMG, NONE, 0},
	{"IMAGE_ATOMIC_FMAX",			0xF07C0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE",				0xF0800000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_CL",				0xF0840000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_D",				0xF0880000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_D_CL",			0xF08C0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_L",				0xF0900000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_B",				0xF0940000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_B_CL",			0xF0980000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_LZ",				0xF09C0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C",				0xF0A00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_CL",			0xF0A40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_D",			0xF0A80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_D_CL",			0xF0AC0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_L",			0xF0B00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_B",			0xF0B40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_B_CL",			0xF0B80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_LZ",			0xF0BC0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_O",				0xF0C00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_CL_O",			0xF0C40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_D_O",			0xF0C80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_D_CL_O",			0xF0CC0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_L_O",			0xF0D00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_B_O",			0xF0D40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_B_CL_O",			0xF0D80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_LZ_O",			0xF0DC0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_O",			0xF0E00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_CL_O",			0xF0E40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_D_O",			0xF0E80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_D_CL_O",		0xF0EC0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_L_O",			0xF0F00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_B_O",			0xF0F40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_B_CL_O",		0xF0F80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_LZ_O",			0xF0FC0000, MIMG, NONE, 0},
	{"IMAGE_GATHER4",				0xF1000000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_CL",			0xF1040000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_L",				0xF1080000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_B",				0xF10C0000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_B_CL",			0xF1100000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_LZ",			0xF1140000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C",				0xF1180000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_CL",			0xF11C0000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_L",			0xF1300000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_B",			0xF1340000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_B_CL",		0xF1380000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_LZ",			0xF13C0000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_O",				0xF1300000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_CL_O",			0xF1440000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_L_O",			0xF1500000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_B_O",			0xF1540000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_B_CL_O",		0xF1580000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_LZ_O",			0xF15C0000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_O",			0xF1600000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_CL_O",		0xF1640000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_L_O",			0xF1700000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_B_O",			0xF1740000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_B_CL_O",		0xF1780000, MIMG, NONE, 0},
	{"IMAGE_GATHER4_C_LZ_O",		0xF17C0000, MIMG, NONE, 0},
	{"IMAGE_GET_LOD",				0xF1800000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_CD",				0xF1A00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_CD_CL",			0xF1A40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_CD",			0xF1A80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_CD_CL",		0xF1AC0000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_CD_O",			0xF1B00000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_CD_CL_O",		0xF1B40000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_CD_O",			0xF1B80000, MIMG, NONE, 0},
	{"IMAGE_SAMPLE_C_CD_CL_O",		0xF1BC0000, MIMG, NONE, 0}

};

const unsigned int isa_instr_count = sizeof(isa_instr_list) / sizeof(isa_instr);